/*
 * Copyright (C) 2012-2019 Japan Smartphone Security Association
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jssec.android.https.imagesearch;

import android.os.AsyncTask;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ProtocolException;
import java.net.MalformedURLException;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSocketFactory;

import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class HttpsImageSearch extends AsyncTask<String, Void, Object> {

    @Override
    protected Object doInBackground(String... params) {
        HttpsURLConnection con1, con2;
        ArrayList<String> imageUrlList = new ArrayList<>();
        byte[] responseArray = null;

        try {
            // --------------------------------------------------------
            // 通信1回目：画像検索する
            // --------------------------------------------------------
            StringBuilder s = new StringBuilder();
            for (String param : params){
                s.append(param);
                s.append('+');
            }
            s.deleteCharAt(s.length() - 1);
            // ★ポイント1★ URIはhttps://で始める
            // ★ポイント2★ 送信データにセンシティブな情報を含めてよい
            // 画像検索文字列 s を（チェックして）送信する…サンプルにつき省略
            String search_url = "https://www.google.com/search?tbm=isch&q=" +
                s.toString();

            // ★ポイント3★ HTTPS接続したサーバーからのデータであっても、受信データの安全性を確認する
            // サンプルにつき省略。「3.2 入力データの安全性を確認する」を参照。
            con1 = connectUrl(search_url);
            BufferedReader in = new BufferedReader(
                                    new InputStreamReader(con1.getInputStream()));
            String inputLine;
            StringBuffer sb = new StringBuffer();
            while ((inputLine = in.readLine()) != null) {
                sb.append(inputLine);
            }
            in.close();
            final String regex = "<img.+?src=\"(.+?)\".+?>";
            Pattern pattern = Pattern.compile(regex);
            Matcher matcher = pattern.matcher(sb.toString());
            while (matcher.find()) {
                if (matcher.group(1).startsWith("https://"))
                    imageUrlList.add(matcher.group(1));
            }
            if (imageUrlList == null || imageUrlList.isEmpty()) {
                return null;
            }

            // --------------------------------------------------------
            // 通信2回目：画像を取得する
            // --------------------------------------------------------

            // ★ポイント1★ URIはhttps://で始める
            // ★ポイント2★ 送信データにセンシティブな情報を含めてよい
            String image_url = imageUrlList.get(1);
            // 画像URL（image_url をチェックして…サンプルにつき省略）送信する
            con2 = connectUrl(image_url);
            checkResponse(con2);
            responseArray = getByteArray(con2);
            if (responseArray == null) {
                return null;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return responseArray;
    }

    private HttpsURLConnection connectUrl(String strUrl) {
        HttpsURLConnection con = null;
        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, null, null);
            SSLSocketFactory sf = new NoSSLv3SocketFactory(sc.getSocketFactory());
            HttpsURLConnection.setDefaultSSLSocketFactory(sf);
            URL url = new URL(strUrl);
            con = (HttpsURLConnection) url.openConnection();
            con.setRequestMethod("GET");
            con.connect();
            String cipher_suite = con.getCipherSuite();
            Certificate[] certs = con.getServerCertificates();
        } catch (SSLException e) {
            // ★ ポイント4 ★ SSLExceptionに対してアプリに適した例外処理を行う
            e.printStackTrace();// サンプルにつき例外処理は省略
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();// 例外処理は省略
        } catch (KeyManagementException e) {
            e.printStackTrace();// 例外処理は省略
        } catch (ProtocolException e) {
            e.printStackTrace();// 例外処理は省略
        } catch (MalformedURLException e) {
            e.printStackTrace();// 例外処理は省略
        } catch (IOException e) {
            e.printStackTrace();// 例外処理は省略
        }
        return con;
    }

    private byte[] getByteArray(HttpsURLConnection con) {
        byte[] buff = new byte[1024];
        byte[] result = null;
        BufferedInputStream inputStream = null;
        ByteArrayOutputStream responseArray = null;
        int length;

        try {
            inputStream = new BufferedInputStream(con.getInputStream());
            responseArray = new ByteArrayOutputStream();

            while ((length = inputStream.read(buff)) != -1) {
                if (length > 0) {
                    responseArray.write(buff, 0, length);
                }
            }
            result = responseArray.toByteArray();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    // 例外処理は省略
                }
            }
            if (responseArray != null) {
                try {
                    responseArray.close();
                } catch (IOException e) {
                    // 例外処理は省略
                }
            }
        }
        return result;
    }

    private void checkResponse(HttpsURLConnection con) throws IOException {
        int statusCode = con.getResponseCode();
        if (HttpsURLConnection.HTTP_OK != statusCode) {
            throw new IOException("HttpStatus: " + statusCode);
        }
    }
}