/*
 * Copyright (C) 2012-2019 Japan Smartphone Security Association
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jssec.notification.visibilityPrivate;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Person;
import android.app.RemoteInput;
import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Icon;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.view.View;

public class VisibilityPrivateNotificationActivity extends Activity {
    /**
     * Private な Notification を表示する
     */
    private final int mNotificationId = 0;

    public static final String DEFAULT_CHANNEL = "default_channel";
    public static final String SENDER_NAME = "Sender Name";

    public static final String REMOTE_REPLY = "remote_reply";
    public static final String reply_choices[] = {"choice1", "choice2", "choice3"};
    public static final String REPLY_LABEL = "input reply";

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (Build.VERSION.SDK_INT >= 26) {
            // api level 26からはNotificationにchannelが必要になる
            NotificationChannel default_channel =
                new NotificationChannel(DEFAULT_CHANNEL,
                                        getString(R.string.notification_channel_default),
                                        NotificationManager.IMPORTANCE_DEFAULT);

            NotificationManager notificationManager =
                (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);
            notificationManager.createNotificationChannel(default_channel);
        }
    }

    public void onSendNotificationClick(View view) {

        // ★ポイント1★  プライベート情報を含んだ通知を行う場合は、公開用（画面ロック時の表示用）のNotification を用意する
        Notification.Builder publicNotificationBuilder;
        if (Build.VERSION.SDK_INT >= 26) {
            publicNotificationBuilder =
                new Notification.Builder(this, DEFAULT_CHANNEL)
                    .setContentTitle("Notification : Public");
        } else {
            publicNotificationBuilder =
                new Notification.Builder(this)
                    .setContentTitle("Notification : Public");
        }

        if (Build.VERSION.SDK_INT >= 21)
            publicNotificationBuilder.setVisibility(Notification.VISIBILITY_PUBLIC);
        // ★ポイント2★ 公開用（画面ロック時の表示用）の Notificationにはプライベート情報を含めない
        publicNotificationBuilder.setContentText("Visibility Public : Omitting sensitive data.");
        publicNotificationBuilder.setSmallIcon(R.drawable.ic_launcher);

        Notification publicNotification = publicNotificationBuilder.build();

        // プライベート情報を含む Notification を作成する
        Notification.Builder privateNotificationBuilder;
        if (Build.VERSION.SDK_INT >= 26) {
            privateNotificationBuilder =
                    new Notification.Builder(this, DEFAULT_CHANNEL)
                            .setContentTitle("Notification : Private");
        } else {
            privateNotificationBuilder =
                    new Notification.Builder(this)
                            .setContentTitle("Notification : Private");
        }

        // ★ポイント3★ Visibility を明示的に Private に設定して、Notification を作成する
        if (Build.VERSION.SDK_INT >= 21)
            privateNotificationBuilder.setVisibility(Notification.VISIBILITY_PRIVATE);
        // ★ポイント4★ Visibility が Private の場合、プライベート情報を含めて通知してもよい
        privateNotificationBuilder.setContentText("Visibility Private : Including user info.");
        privateNotificationBuilder.setSmallIcon(R.drawable.ic_launcher);
        // VisibilityがPrivateのNotificationを利用する場合、
        // VisibilityをPublicにした公開用のNotificationを合わせて設定する
        if (Build.VERSION.SDK_INT >= 21)
            privateNotificationBuilder.setPublicVersion(publicNotification);

        if (Build.VERSION.SDK_INT >= 28) {
            // Notification.MessagingStyle.Message.setData()により
            // リソースのsample_picture.pngをNotificationに表示する
            int resourceId = R.drawable.sample_picture_small;
            Uri imageUri =
                Uri.parse("android.resource://" +
                          getApplicationContext().getPackageName() + "/" + resourceId);

            Person sender = new Person.Builder()
                                .setName(SENDER_NAME)
                                .setIcon(null)
                                .build();

            // Notification.MessagingStyle.Messageを用意しsetData()で画像を設定する
            Notification.MessagingStyle.Message message =
                new Notification.MessagingStyle.Message("Sample Picture", 0, sender)
                    .setData("image/png", imageUri);

            // Notification.MessagingStyleを用意し画像を設定した
            // Notification.MessagingStyle.Messageを設定する
            Notification.MessagingStyle message_style =
                new Notification.MessagingStyle(sender)
                    .addMessage(message);

            // Notification.MessagingStyleをNotificationに設定する
            privateNotificationBuilder.setStyle(message_style);
        }

        if (Build.VERSION.SDK_INT >= 28) {
            // RemoteInput.Builder.setChoices()によりNotificationにリプライの選択肢を表示する
            Intent intent =
                new Intent( getApplicationContext(), NotificationReceiver.class);

            PendingIntent pendingIntent =
                PendingIntent.getBroadcast(this, 0, intent, 0);

            RemoteInput remoteInput = new RemoteInput.Builder(REMOTE_REPLY)
                        .setLabel(REPLY_LABEL)
                        .setChoices(reply_choices)
                        .build();

            Icon icon = Icon.createWithResource(this, R.drawable.ic_launcher);

            Notification.Action actionReply =
                new Notification.Action.Builder(icon, REPLY_LABEL, pendingIntent)
                    .addRemoteInput(remoteInput)
                    .setSemanticAction(Notification.Action.SEMANTIC_ACTION_REPLY)
                    .build();

            privateNotificationBuilder.addAction(actionReply);
        }

        Notification privateNotification = privateNotificationBuilder.build();

        // 本サンプルでは実装していないが、Notificationでは setContentIntent(PendingIntent intent)
        // を使いNotificationをクリックした際にIntentが送信されるように実装することが多い。
        // このときに設定するIntentは、呼び出すコンポーネントの種類に合わせて、
        // 安全な方法で呼び出すことが必要である（例えば、明示的Intentを使うなど）
        // 各コンポーネントの安全な呼び出し方法は以下の項目を参照のこと
        // 4.1. Activity を作る・利用する
        // 4.2. Broadcast を受信する・送信する
        // 4.4. Service を作る・利用する

        NotificationManager notificationManager =
                (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);
        notificationManager.notify(mNotificationId, privateNotification);
    }
}

